<?php

namespace Drupal\session_inspector;

/**
 * A utility class to unserialize PHP session data.
 *
 * Adapted from https://www.php.net/manual/en/function.session-decode.php#108037
 *
 * @package Drupal\session_inspector
 */
class SessionUnserializer {

  /**
   * Method to unserialize a string of PHP session data.
   *
   * This method will detect what type of session data is stored and hand off
   * the extraction to other methods.
   *
   * @param string $session_data
   *   The session data.
   *
   * @return array
   *   The unserialized array of session data.
   *
   * @throws \Exception
   */
  public static function unserialize($session_data) {
    $method = ini_get("session.serialize_handler");

    switch ($method) {
      case "php":
        return self::unserializePhp($session_data);

      case "php_binary":
        return self::unserializePhpBinary($session_data);

      default:
        throw new \Exception("Unsupported session.serialize_handler: " . $method . ". Supported: php, php_binary");
    }
  }

  /**
   * Extract the session data from a PHP session string.
   *
   * @param string $session_data
   *   The session data.
   *
   * @return array
   *   The unserialized array of session data.
   */
  private static function unserializePhp($session_data) {
    // Store the current session data in a variable.
    $session = $_SESSION;

    // Destroy the current session.
    session_destroy();

    // Create a new session and populate it with the passed session data.
    session_start();
    session_decode($session_data);

    // Extract the decoded session data.
    $return_data = $_SESSION;

    // Destroy our temporary session.
    session_destroy();

    // Create a new session for the user and populate it with their current
    // session data.
    session_start();
    $_SESSION = $session;

    // Return the extracted session data.
    return $return_data;
  }

  /**
   * Extract the session data from a PHP Binary session string.
   *
   * @param string $session_data
   *   The session data.
   *
   * @return array
   *   The unserialized array of session data.
   *
   * @throws \Exception
   */
  private static function unserializePhpBinary($session_data) {
    $return_data = [];
    $offset = 0;

    while ($offset < strlen($session_data)) {
      $num = ord($session_data[$offset]);
      $offset += 1;
      $varname = substr($session_data, $offset, $num);
      $offset += $num;
      $data = unserialize(substr($session_data, $offset), ['allowed_classes' => FALSE]);
      $return_data[$varname] = $data;
      $offset += strlen(serialize($data));
    }

    return $return_data;
  }

}
